# [Full API](@id Full-API)

```@raw html
<style>
    .content table td {
        padding-top: 0 !important;
        padding-bottom: 0 !important;
    }
</style>
```

## States

Stabilizer states can be represented with the [`Stabilizer`](@ref), [`Destabilizer`](@ref), [`MixedStabilizer`](@ref), and [`MixedDestabilizer`](@ref) tableau data structures. You probably want to use [`MixedDestabilizer`](@ref) which supports the widest set of operations.

Moreover, a [`MixedDestabilizer`](@ref) can be stored inside a [`Register`](@ref) together with a set of classical bits in which measurement results can be written.

There are [convenience constructors for common types of states and operators](@ref Useful-States-and-Operators).

## [Operations](@id all-operations)

All of these can be applied on a state with the [`apply!`](@ref) function. Whether they are deterministic and their computational complexity is listed in the table below. A list of lower-level functions for more control over how an operation is performed is also given.

| Type | Deterministic | 𝒪(nˣ) | Low-level functions 
|:--|:-:|---|---|
|`AbstractOperation                      `|  |   |                        |
|` ├─ AbstractCliffordOperator           `|  |   |                        |
|` │   ├─ AbstractSymbolicOperator       `|  |   |                        |
|` │   │   ├─ AbstractSingleQubitOperator`|  |   |                        |
|` │   │   │   ├─ SingleQubitOperator    `|✔️ | n |                        |
|` │   │   │   ├─ sHadamard              `|✔️ | n |                        |
|` │   │   │   ├─ sId1                   `|✔️ | n |                        |
|` │   │   │   ├─ sInvPhase              `|✔️ | n |                        |
|` │   │   │   ├─ sPhase                 `|✔️ | n |                        |
|` │   │   │   ├─ sX                     `|✔️ | n |                        |
|` │   │   │   ├─ sY                     `|✔️ | n |                        |
|` │   │   │   └─ sZ                     `|✔️ | n |                        |
|` │   │   └─ AbstractTwoQubitOperator   `|  |   |                        |
|` │   │       ├─ sCNOT                  `|✔️ | n |                        |
|` │   │       ├─ sCPHASE                `|✔️ | n |                        |
|` │   │       └─ sSWAP                  `|✔️ | n |                        |
|` │   │                                 `|  |   |                        |
|` │   ├─ CliffordOperator               `|✔️ | n³|                        |
|` │   ├─ PauliOperator                  `|✔️ | n²|                        |
|` │   └─ SparseGate                     `|✔️ |kn²|                        |
|` ├─ AbstractMeasurement                `|  |   |                        |
|` │   ├─ PauliMeasurement               `|❌ | n²| [`project!`](@ref), [`projectrand!`](@ref) |
|` │   ├─ sMX                            `|❌ | n²| [`projectX!`](@ref)    |
|` │   ├─ sMY                            `|❌ | n²| [`projectY!`](@ref)    |
|` │   └─ sMZ                            `|❌ | n²| [`projectZ!`](@ref)    |
|` │                                     `|  |   |                        |
|` └─ Reset                              `|✔️ |kn²| [`reset_qubits!`](@ref)|

## Autogenerated API list

```@autodocs
Modules = [QuantumClifford]
Private = false
```
